import os
import shutil
import urllib.request
import zipfile
from html import unescape as html_unescape
from logging import debug, warning, error, info
from os.path import join
from subprocess import STDOUT, PIPE
from xml.sax.saxutils import escape as xml_escape

import ruamel.std.zipfile as zip_utils
from kindlecomicconverter.comic2ebook import main as kcc_c2e
from psutil import Popen

from .chapter import Chapter
from .comic import Comic


def escape_file_path(path: str):
    return path.replace(os.sep, '|')


def chapter_temp_directory_name(chapter):
    return chapter_pattern(chapter.chapter, escape_file_path(chapter.title))


def volume_pattern(title, volume: str):
    if volume.isnumeric():
        return '{} - Volume {}'.format(title, volume)

    if volume.strip() != '':
        return '{} - {}'.format(title, volume)

    return '{}'.format(title)


def chapter_pattern(chapter_number, chapter_title: str):
    if chapter_title is None or chapter_title.strip() == '':
        return 'Chapter {}'.format(chapter_number)

    return 'Chapter {}: {}'.format(chapter_number, chapter_title)


def read_files_as_chapters(files):
    chapters = []

    for file in files:
        with zipfile.ZipFile(file, "r") as zip_file:
            if 'info.json' in zip_file.namelist():
                info_file = [file for file in zip_file.namelist() if file == 'info.json'][0]
                zip_file_data = zip_file.read(info_file).decode("utf-8")

                chapters.append(Chapter.from_file(zip_file_data, file))

            else:
                chapter_info = zip_file.filename.rsplit(os.sep, 1)[1].replace('.zip', '').rsplit('-', 1)
                chapter_number = ''.join([letter for letter in chapter_info[0] if letter.isnumeric()]).strip('0')
                chapter_language = chapter_info[1]
                chapters.append(
                    Chapter(volume='Single Volume',
                            chapter=chapter_number,
                            title='',
                            language=chapter_language,
                            publisher='',
                            local_path=file
                            )
                )

    return chapters


def read_chapters_as_volumes(chapters):
    volumes = {}

    for chapter in chapters:
        if chapter.volume not in volumes:
            volumes[chapter.volume] = []

        volumes_chapters = volumes[chapter.volume]
        volumes_chapters.append(chapter)

    return volumes


def escape_text(text):
    return xml_escape(html_unescape(text))


def build_comic_info_author_xml(authors: list):
    authors_str = escape_text(', '.join(authors))

    return '<Writer>{}</Writer>'.format(authors_str) if authors != '' else ''


def build_comic_info_xml(comic: Comic, volume):
    series_val = escape_text(volume_pattern(comic.title, volume))
    summary_val = escape_text(comic.description)

    series_tag = '<Series>{}</Series>'.format(series_val) if series_val != '' else ''
    summary_tag = '<Summary>{}</Summary>'.format(summary_val) if summary_val != '' else ''

    return '''<?xml version="1.0" encoding="utf-8"?>
<ComicInfo xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
{}
{}
{}
</ComicInfo>'''.format(series_tag, summary_tag, build_comic_info_author_xml(comic.authors))


def extract_volume_pages(settings, comic, volumes):
    for volume, chapters in volumes.items():
        volume_directory = settings.volume_temp_directory(comic, volume)
        os.mkdir(volume_directory)

        debug('Extracting pages for "{}"...'.format(volume_pattern(comic.title, volume)))

        for chapter in chapters:
            chapter_directory = join(volume_directory, chapter_temp_directory_name(chapter))
            os.mkdir(chapter_directory)

            with zipfile.ZipFile(chapter.local_path, "r") as zip_file:
                for name in zip_file.namelist():
                    with open(join(chapter_directory, name), 'wb') as copy_file:
                        copy_file.write(zip_file.read(name))

            debug('Extracting {}'.format(chapter_pattern(chapter.chapter, chapter.title)))

        # build ComicInfo.xml file
        with open(join(volume_directory, 'ComicInfo.xml'), 'w') as comic_info_xml:
            comic_info_xml.write(build_comic_info_xml(comic, volume))


def get_available_file(file):
    if not os.path.exists(file):
        return file

    file_name = file.rsplit('.', 1)
    file_pattern = os.path.join(file_name[0] + ' ({}).' + file_name[1])
    postfix = 0

    while os.path.exists(file_pattern.format(postfix)):
        postfix += 1

    return file_pattern.format(postfix)


def assemble_volumes(settings, comic, volumes):
    assembled = {}

    for volume in volumes.keys():
        final_output_file = join(settings.output,
                                 '{}.{}'.format(volume_pattern(comic.title, volume), settings.comic_format.lower()))

        if os.path.exists(final_output_file):
            info('File "{}" already exists'.format(final_output_file))

            if settings.skip_existing:
                info('Skipping...')
                continue

            if settings.overwrite_existing:
                debug('Overwriting...')
                os.remove(final_output_file)

        volume_temp_data = settings.volume_temp_directory(comic, volume)
        os.path.isdir(settings.output) or os.makedirs(settings.output)

        output_file = join(settings.output,
                           '{}.{}'.format(volume_pattern(comic.title, volume),
                                          settings.device_comic_format_kcc.lower()))

        output_file = get_available_file(output_file)

        args = [
            settings.device_profile_kcc_param,
            settings.comic_style_param,
            settings.low_quality_param,
            settings.output_param,
            '--title={}'.format(volume_pattern(comic.title, volume)),
            settings.device_comic_format_kcc_param,
            settings.upscale_param,
            settings.stretch_param,
            settings.grayscale_param,
            '--output={}'.format(output_file),
            volume_temp_data,
        ]
        args = [arg for arg in args if arg != '']  # cleanup kcc args
        debug("kcc args: {}".format(' '.join(args)))

        success = kcc_c2e(args) == 0  # assemble success

        if success:
            assembled[volume] = output_file

    return assembled


def chapter_publishers_and_languages(chapters):
    return chapter_publishers(chapters), chapter_languages(chapters)


def chapter_publishers(chapters):
    return set([chapter.publisher for chapter in chapters])


def chapter_languages(chapters):
    return set([chapter.language for chapter in chapters])


def fill_metadata(settings, comic, chapters, assembled_ebooks):
    publishers, languages = chapter_publishers_and_languages(chapters)

    for volume, ebook_file in assembled_ebooks.items():
        if volume not in comic.volume_covers:
            warning('Volume "{}" not found in the list of covers...'.format(volume))
            continue

        cloud_cover_url = comic.volume_covers[volume]
        response = urllib.request.urlopen(cloud_cover_url)

        local_cover_path = join(settings.temp_directory(comic), volume_pattern(comic.title, volume), 'cover')

        with open(local_cover_path, 'wb') as local_cover:
            local_cover_bytes = response.read()
            local_cover.write(local_cover_bytes)

        zip_cover_path = join('OEBPS', 'Images', 'cover.jpg')
        zip_utils.delete_from_zip_file(ebook_file, file_names=zip_cover_path)

        with zipfile.ZipFile(ebook_file, 'a') as ebook:
            ebook.write(local_cover_path, zip_cover_path)


def convert_to_mobi(assembled_ebooks):
    converted_ebooks = {}

    kindle_gen_exists = Popen('kindlegen', stdout=PIPE, stderr=STDOUT, stdin=PIPE, shell=True)
    kindle_gen_exists.communicate()

    if kindle_gen_exists.returncode == 0:
        for volume, ebook_file in assembled_ebooks.items():
            info('Converting file "{}" to MOBI...'.format(ebook_file))

            kindle_gen_convert = Popen(
                'kindlegen "{}" -c2'.format(ebook_file),
                stdout=PIPE,
                stderr=STDOUT,
                stdin=PIPE,
                shell=True
            )
            kindle_gen_convert.communicate()

            if kindle_gen_convert.returncode != 0:
                error('Error to convert file "{}" to MOBI'.format(ebook_file))
                continue

            os.remove(ebook_file)
            converted_ebooks[volume] = os.path.join(ebook_file, str(ebook_file).replace('.epub', '.mobi'))
    else:
        warning('KindleGen not found!')

    return converted_ebooks


def clean_temporary_data(temp_directory, assembled_ebooks=None, force_clean=False):
    if assembled_ebooks is None:
        assembled_ebooks = {}

    safe_to_remove_temp_dir = force_clean or len(
        [ebook for ebook in assembled_ebooks.values() if ebook in temp_directory]) == 0

    if safe_to_remove_temp_dir:
        shutil.rmtree(temp_directory)

    else:
        warning(
            'Some books are in the same folder used as auxiliary folder! So the cache cleaning is your responsibility')



import json
from dataclasses import dataclass


@dataclass
class Chapter:
    volume: str
    chapter: str
    title: str
    language: str
    publisher: str
    local_path: str

    @staticmethod
    def from_file(json_string, local_path):
        json_object = json.loads(json_string)

        return Chapter(
            volume=json_object['volume'],
            chapter=json_object['chapter'],
            title=json_object['title'],
            language=json_object['language'],
            publisher=json_object['publisher'],
            local_path=local_path
        )



import sys
import zipfile
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter as Formatter
from logging import fatal
from os import path, listdir
from os.path import isfile, join

import ruamel.std.zipfile as zip_utils


def build_argument_parser():
    parser = ArgumentParser(
        prog='metadata-injector',
        description=(
            'metadata-injector is a comic_builder auxiliary tool, made to insert metadata in comic chapters'
        ),
        formatter_class=Formatter
    )
    parser.add_argument(
        'chapter_dir',
        metavar='CHAPTER_DIR',
        help='Chapter directory with zip files that will receive the metadata in the same path with the same name',
    )
    return parser


def main():
    # Read arguments
    argument_parser = build_argument_parser()
    args = argument_parser.parse_args()

    chapter_dir = args.chapter_dir

    if not path.isdir(chapter_dir):
        fatal('Directory "{}" does not exists'.format(chapter_dir))
        sys.exit(1)

    zip_files = [join(chapter_dir, file) for file in listdir(chapter_dir) if file.endswith('.zip')]
    metadata_chapters = {
        file: file.rstrip('.zip') + '.json'
        for file in zip_files if isfile(file.rstrip('.zip') + '.json')
    }

    for chapter, metadata in metadata_chapters.items():
        with zipfile.ZipFile(chapter, 'r') as zip_file:
            contains_metadata = len([file for file in zip_file.namelist() if file == 'info.json']) > 0

        if contains_metadata:
            zip_utils.delete_from_zip_file(chapter, file_names='info.json')

        with zipfile.ZipFile(chapter, 'a') as zip_file:
            zip_file.write(metadata, 'info.json')


if __name__ == '__main__':
    main()



import hashlib
import os
import sys
import tarfile
import urllib.request
from argparse import ArgumentParser
from logging import warning, fatal
from os import path
from os.path import join
from tempfile import gettempdir as temporary_directory


def kindlegen_linux_install(parser):
    # prepare temporary folder and file
    temp_folder = join(temporary_directory(), 'kindlegen-installer')
    kindlegen_file_path = join(temp_folder, 'kindlegen_linux_2.6_i386_v2_9.tar.gz')
    kindlegen_file_dest = '{}/.local/bin'.format(path.expanduser('~')) if parser.user else '/usr/local/bin'

    # check file integrity
    expected_hash = '5bd4928785ee06d92de1991a34029ffe343c087b4311f52c83df8d821d826b2e42f29e3047c0536140457fd478d300f621648f9eaef3a6568431a791e8147937'

    if path.isfile(kindlegen_file_path):
        with open(kindlegen_file_path, 'rb') as kindlegen_file:
            file_hash = hashlib.sha512(kindlegen_file.read()).hexdigest()

        if file_hash != expected_hash:
            os.remove(kindlegen_file_path)

    path.isdir(temp_folder) or os.makedirs(temp_folder)

    # get file from internet
    if not path.isfile(kindlegen_file_path):
        kindlegen_url = 'https://archive.org/download/kindlegen2.9/kindlegen_linux_2.6_i386_v2_9.tar.gz'
        response = urllib.request.urlopen(kindlegen_url)

        # write file in a temporary file
        with open(kindlegen_file_path, 'wb') as kindlegen_file:
            kindlegen_file.write(response.read())

    # extract kindlegen binary
    with tarfile.open(kindlegen_file_path, 'r') as kindlegen_file:
        try:
            path.isdir(kindlegen_file_dest) or os.makedirs(kindlegen_file_dest)

            kindlegen_file.extract('kindlegen', kindlegen_file_dest)

        except PermissionError:
            warning('User has no permission to create the binary file in "{}"'.format(kindlegen_file_dest))
            warning('Run the command with super user privileges (e.g. sudo) or --user to use the user bin folder')


def kindlegen_mac_install():
    warning('MAC install pending of implementation...')
    sys.exit(1)


def kindlegen_windows_install():
    warning('WIN install pending of implementation...')
    sys.exit(1)


def main():
    # Define and read arguments
    parser = ArgumentParser()
    parser.add_argument('--user', help='Install binary in the user binary folder, instead the system binary folder',
                        action='store_true')
    parser = parser.parse_args()

    if sys.platform.startswith('linux'):
        kindlegen_linux_install(parser)

    elif sys.platform.startswith('mac'):
        kindlegen_mac_install()

    elif sys.platform.startswith('win'):
        kindlegen_windows_install()

    else:
        fatal('Unknown platform "{}"...'.format(sys.platform))
        sys.exit(1)


if __name__ == '__main__':
    main()



import os
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter as Formatter


def build_argument_parser():
    parser = ArgumentParser(
        prog='comic_builder',
        description=(
            'comic_builder is a manga-py auxiliary tool, made for organize and join comic chapters{}'
            'Links:{}'
            '  comic_builder{}'
            '    * source-code.: https://github.com/eduhoribe/comic-builder{}'
            '  manga-py{}'
            '    * site........: https://manga-py.com/manga-py{}'
            '    * source-code.: https://github.com/manga-py/manga-py{}'
        ).format(os.linesep, os.linesep, os.linesep, os.linesep, os.linesep, os.linesep, os.linesep),
        epilog=(
            'So, that is how manga-py-assemble can be executed to join yours favourite comics.{}'
            'Enjoy! ;)'
        ).format(os.linesep),
        formatter_class=Formatter
    )

    required_group = parser.add_argument_group('Required options')
    required_group.add_argument('comic_path', metavar='COMIC_PATH', help='Comic path to be assembled')

    archive_group = parser.add_argument_group('Archive options')
    archive_group.add_argument('--output-format',
                               help='Output format. (Default: %(default)s)',
                               choices=['MOBI', 'EPUB'],
                               default='MOBI'
                               )
    archive_group.add_argument('--output', help='Output folder. (Default: COMIC_PATH/assembled)')
    archive_group.add_argument('--aux-parent-folder',
                               help='Auxiliary parent folder, who will store the images temporally. '
                                    '(Default: System temporary folder)')
    archive_subgroup = archive_group.add_mutually_exclusive_group()
    archive_subgroup.add_argument('--overwrite-existing', help='Writes over existing files', action='store_true')
    archive_subgroup.add_argument('--skip-existing', help='Skip existing files', action='store_true')

    metadata_group = parser.add_argument_group('Metadata options')
    metadata_group.add_argument('--metadata',
                                help='Path to comic metadata file. (Default: COMIC_PATH/info.json) '
                                     'The file must follow the sample in '
                                     'https://github.com/eduhoribe/comic-builder/blob/goshujin-sama/samples/comic-metadata-sample.json')
    metadata_group.add_argument('--title', help='Comic Title')
    metadata_group.add_argument('--authors', help='Comic Authors', nargs='+')

    image_group = parser.add_argument_group('Image processing options')
    image_group.add_argument('--device-profile',
                             help='Device profile. (Default: %(default)s)',
                             choices=['K1', 'K2', 'K34', 'K578', 'KDX', 'KPW', 'KV', 'KO', 'KoMT', 'KoG', 'KoGHD',
                                      'KoA',
                                      'KoAHD', 'KoAH2O', 'KoAO', 'KoF'],
                             default='KO',
                             )
    image_group.add_argument('--comic-style', help='Comic style. (Default: %(default)s)',
                             choices=['DEFAULT', 'MANGA', 'WEBTOON'], default='DEFAULT')
    image_group.add_argument('--no-upscale',
                             help='Do not resize images smaller than device resolution. (NOT RECOMMENDED!)',
                             action="store_true")
    image_group.add_argument('--stretch', help='Stretch images to device resolution. May lose original proportion!',
                             action="store_true")
    image_group.add_argument('--grayscale', help='Apply grayscale filter in all pages', action="store_true")
    image_group.add_argument('--low-quality', help='Omits -hq parameter for kcc', action="store_true")

    misc_group = parser.add_argument_group('Miscellaneous options')
    misc_group.add_argument('--debug', help='Enable debug mode', action="store_true")

    return parser



import sys
import zipfile
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter as Formatter
from logging import fatal
from os import path, listdir
from os.path import join


def build_argument_parser():
    parser = ArgumentParser(
        prog='metadata-ejector',
        description=(
            'metadata-ejector is a comic_builder auxiliary tool, made to export metadata in comic chapters'
        ),
        formatter_class=Formatter
    )
    parser.add_argument(
        'chapter_dir',
        metavar='CHAPTER_DIR',
        help='Chapter directory with zip files that will provide the metadata',
    )
    return parser


def main():
    # Read arguments
    argument_parser = build_argument_parser()
    args = argument_parser.parse_args()

    chapter_dir = args.chapter_dir

    if not path.isdir(chapter_dir):
        fatal('Directory "{}" does not exists'.format(chapter_dir))
        sys.exit(1)

    zip_files = [join(chapter_dir, file) for file in listdir(chapter_dir) if file.endswith('.zip')]
    metadata_chapters = {
        file: file.rstrip('.zip') + '.json'
        for file in zip_files
    }

    for chapter, metadata in metadata_chapters.items():
        with zipfile.ZipFile(chapter, 'r') as zip_file:
            contains_metadata = len([file for file in zip_file.namelist() if file == 'info.json']) > 0

        if contains_metadata:
            with zipfile.ZipFile(chapter, 'r') as zip_file:
                metadata_info = zip_file.read('info.json').decode('utf-8')

            with open(metadata, 'w') as metadata_file:
                metadata_file.write(metadata_info)


if __name__ == '__main__':
    main()



import random
from dataclasses import dataclass
from os import listdir
from os.path import isfile, join
from tempfile import gettempdir as temporary_directory

from . import utils


@dataclass
class Settings:
    random_id: str
    comic_path: str
    device_profile: str
    comic_style: str
    low_quality: bool
    output: str
    comic_format: str
    no_upscale: bool
    stretch: bool
    grayscale: bool
    auxiliary_folder: str
    overwrite_existing: bool
    skip_existing: bool
    debug_enabled = bool

    def __init__(self, args):
        self.random_id = hex(hash(random.random()))

        self.comic_path = args.comic_path
        self.metadata_path = self.__prepare_metadata_path__(args.metadata)
        self.device_profile = args.device_profile
        self.comic_style = args.comic_style
        self.low_quality = args.low_quality
        self.output = self.__prepare_output__(args.output)
        self.comic_format = args.output_format
        self.no_upscale = args.no_upscale
        self.stretch = args.stretch
        self.grayscale = args.grayscale
        self.auxiliary_folder = args.aux_parent_folder
        self.overwrite_existing = args.overwrite_existing
        self.skip_existing = args.skip_existing
        self.debug_enabled = args.debug

    @property
    def device_comic_format_kcc(self):
        return 'EPUB'

    @property
    def device_comic_format_kcc_param(self):
        return '--format={}'.format(self.device_comic_format_kcc)

    @property
    def device_profile_kcc_param(self):
        return '--profile={}'.format(self.device_profile)

    @property
    def comic_style_param(self):
        if self.comic_style == 'MANGA':
            return '--manga-style'

        elif self.comic_style == 'WEBTOON':
            return '--webtoon'

        return ''

    @property
    def low_quality_param(self):
        return '' if self.low_quality else '--hq'

    @property
    def output_param(self):
        return '--output={}'.format(self.output)

    @property
    def upscale_param(self):
        return '' if self.no_upscale else '--upscale'

    @property
    def stretch_param(self):
        return '--stretch' if self.stretch else ''

    @property
    def grayscale_param(self):
        return '' if self.grayscale else '--forcecolor'

    def temp_directory(self, comic):
        return '{}/comic_builder/{}_{}'.format(
            self.auxiliary_folder if self.auxiliary_folder is not None else temporary_directory(),
            self.random_id,
            comic.title
        )

    def volume_temp_directory(self, comic, volume):
        return join(self.temp_directory(comic), utils.escape_file_path(utils.volume_pattern(comic.title, volume)))

    def read_files(self):
        return [join(self.comic_path, file) for file in listdir(self.comic_path)
                if isfile(join(self.comic_path, file)) and file.endswith('.zip')]

    def __prepare_output__(self, output):
        return output if output is not None else join(self.comic_path, 'assembled')

    def __prepare_metadata_path__(self, metadata_path):
        return metadata_path if metadata_path is not None else join(self.comic_path, 'info.json')



import json
import sys
from logging import fatal
from os import path

from .comic import Comic


class MetadataParser:

    @staticmethod
    def parse(metadata_file):
        if path.isfile(metadata_file):
            with open(metadata_file, 'r') as comic_info_json:
                comic_info_json = json.load(comic_info_json)
        else:
            fatal('Metadata file "{}" not found'.format(metadata_file))
            sys.exit(1)

        return MetadataParser.__read_json_info__(comic_info_json)

    @staticmethod
    def __read_json_info__(comic_json):
        return Comic(
            title=comic_json['title'],
            description=comic_json['description'],
            authors=comic_json['authors'],
            sauce=comic_json['sauce'],
            volume_covers=comic_json['volume_covers']
        )



from dataclasses import dataclass


@dataclass
class Comic:
    title: str
    description: str
    authors: list
    sauce: str
    volume_covers: dict

    def overwrite_user_metadata(self, args):
        self.title = args.title if args.title is not None else self.title
        self.authors = [args.authors] if args.authors is not None else self.authors



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = [req.strip() for req in fh.readlines()]

setuptools.setup(
    name="comic-builder",
    version="0.0.23",
    author="Eduwardo Horibe",
    author_email="eduhoribe@gmail.com",
    description="Build e-book files from comic images",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/eduhoribe/comic-builder",
    packages=setuptools.find_packages(exclude=(
        'tests',
    )),
    keywords=['comic', 'manga', 'manga-py', 'ebook', 'epub', 'mobi', 'kindle'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'comic-builder = comic_builder.__main__:main',
            'comic-metadata-injector = comic_metadata.metadata_injector:main',
            'comic-metadata-ejector = comic_metadata.metadata_ejector:main',
            'kindlegen_installer = kindlegen_installer.kindlegen_installer:main',
        ]
    },
)



import logging
import os
import sys
from logging import fatal, debug, info, warning
from os import path

from . import utils
from .argument_parser import build_argument_parser
from .metadata_parser import MetadataParser
from .settings import Settings


def main():
    # Read arguments
    argument_parser = build_argument_parser()
    args = argument_parser.parse_args()

    # Define settings and log level
    settings = Settings(args)
    logging.getLogger().setLevel(logging.DEBUG if settings.debug_enabled else logging.INFO)
    debug(args)
    debug(settings)

    if not path.isdir(settings.comic_path):
        fatal('Comic path "{}" does not exist'.format(path.abspath(settings.comic_path)))
        sys.exit(1)

    # Read comic info
    comic = MetadataParser.parse(settings.metadata_path)
    comic.overwrite_user_metadata(args)
    debug(comic)

    # Prepare temporary directory
    temp_directory = settings.temp_directory(comic)
    if path.isdir(temp_directory):
        fatal('Directory {} exists...somehow. Stopping right here to prevent a bigger mess'.format(temp_directory))
        sys.exit(1)

    debug('Using temporary directory "{}"'.format(temp_directory))
    os.makedirs(temp_directory)

    # Find files
    files = settings.read_files()

    # Search for chapters
    chapters = utils.read_files_as_chapters(files)
    if len(chapters) == 0:
        warning('No chapter saved was found!')

        utils.clean_temporary_data(temp_directory, force_clean=True)

        sys.exit(1)

    # Merge chapters in volumes
    volumes = utils.read_chapters_as_volumes(chapters)

    # Extract chapter pages
    utils.extract_volume_pages(settings, comic, volumes)

    # Assemble volumes in ebook format
    assembled_ebooks = utils.assemble_volumes(settings, comic, volumes)
    if len(assembled_ebooks) <= 0:
        warning('No file was assembled!')
        sys.exit(0)

    # Fill metadata
    utils.fill_metadata(settings, comic, chapters, assembled_ebooks)

    # Convert to MOBI
    if settings.comic_format == 'MOBI':
        assembled_ebooks = utils.convert_to_mobi(assembled_ebooks)

    # Show results
    info('Finished!')
    if len(assembled_ebooks) > 0:
        info('Files can be found in: {}'.format(path.abspath(settings.output)))
    else:
        warning('No file was assembled!')

    # Remove temporary data
    utils.clean_temporary_data(temp_directory, assembled_ebooks)


if __name__ == '__main__':
    main()